@@ -35,9 +35,11 @@
 #include "libbtrfsutil/btrfsutil.h"
 #include "kernel-lib/list.h"
 #include "kernel-lib/sizes.h"
-#include "kernel-shared/ctree.h"
-#include "kernel-shared/volumes.h"
 #include "kernel-lib/list_sort.h"
+#include "kernel-shared/uapi/btrfs.h"
+#include "kernel-shared/ctree.h"
+#include "kernel-shared/compression.h"
+#include "kernel-shared/volumes.h"
 #include "kernel-shared/disk-io.h"
 #include "common/defs.h"
 #include "common/internal.h"
@@ -53,9 +55,9 @@
 #include "common/parse-utils.h"
 #include "common/string-utils.h"
 #include "common/filesystem-utils.h"
+#include "common/format-output.h"
 #include "cmds/commands.h"
 #include "cmds/filesystem-usage.h"
-#include "ioctl.h"
 
 /*
  * for btrfs fi show, we maintain a hash of fsids we've already printed.
@@ -74,10 +76,12 @@ static const char * const cmd_filesystem_df_usage[] = {
 	"Show space usage information for a mount point",
 	"",
 	HELPINFO_UNITS_SHORT_LONG,
+	HELPINFO_INSERT_GLOBALS,
+	HELPINFO_INSERT_FORMAT,
 	NULL
 };
 
-static void print_df(int fd, struct btrfs_ioctl_space_args *sargs, unsigned unit_mode)
+static void print_df_text(int fd, struct btrfs_ioctl_space_args *sargs, unsigned unit_mode)
 {
 	u64 i;
 	struct btrfs_ioctl_space_info *sp = sargs->spaces;
@@ -98,6 +102,44 @@ static void print_df(int fd, struct btrfs_ioctl_space_args *sargs, unsigned unit
 	}
 }
 
+static const struct rowspec filesystem_df_rowspec[] = {
+	{ .key = "bg-type", .fmt = "%s", .out_json = "bg-type" },
+	{ .key = "bg-profile", .fmt = "%s", .out_json = "bg-profile" },
+	{ .key = "total", .fmt = "%llu", .out_json = "total" },
+	{ .key = "used", .fmt = "%llu", .out_json = "used" },
+	{ .key = "zone_unusable", .fmt = "%llu", .out_json = "zone_unusable" },
+	ROWSPEC_END
+};
+
+static void print_df_json(int fd, struct btrfs_ioctl_space_args *sargs)
+{
+	struct format_ctx fctx;
+	u64 i;
+	struct btrfs_ioctl_space_info *sp = sargs->spaces;
+	u64 unusable;
+	bool ok;
+
+	fmt_start(&fctx, filesystem_df_rowspec, 1, 0);
+	fmt_print_start_group(&fctx, "filesystem-df", JSON_TYPE_ARRAY);
+
+	for (i = 0; i < sargs->total_spaces; i++, sp++) {
+		unusable = device_get_zone_unusable(fd, sp->flags);
+		ok = (unusable != DEVICE_ZONE_UNUSABLE_UNKNOWN);
+
+		fmt_print_start_group(&fctx, NULL, JSON_TYPE_MAP);
+		fmt_print(&fctx, "bg-type", btrfs_group_type_str(sp->flags));
+		fmt_print(&fctx, "bg-profile", btrfs_group_profile_str(sp->flags));
+		fmt_print(&fctx, "total", sp->total_bytes);
+		fmt_print(&fctx, "used", sp->used_bytes);
+		if (ok)
+			fmt_print(&fctx, "zone_unusable", unusable);
+		fmt_print_end_group(&fctx, NULL);
+	}
+
+	fmt_print_end_group(&fctx, "filesystem-df");
+	fmt_end(&fctx);
+}
+
 static int cmd_filesystem_df(const struct cmd_struct *cmd,
 			     int argc, char **argv)
 {
@@ -124,7 +166,10 @@ static int cmd_filesystem_df(const struct cmd_struct *cmd,
 	ret = get_df(fd, &sargs);
 
 	if (ret == 0) {
-		print_df(fd, sargs, unit_mode);
+		if (bconf.output_format == CMD_FORMAT_JSON)
+			print_df_json(fd, sargs);
+		else
+			print_df_text(fd, sargs, unit_mode);
 		free(sargs);
 	} else {
 		errno = -ret;
@@ -135,7 +180,7 @@ static int cmd_filesystem_df(const struct cmd_struct *cmd,
 	close_file_or_dir(fd, dirstream);
 	return !!ret;
 }
-static DEFINE_SIMPLE_COMMAND(filesystem_df, "df");
+static DEFINE_COMMAND_WITH_FLAGS(filesystem_df, "df", CMD_FORMAT_JSON);
 
 static int match_search_item_kernel(u8 *fsid, char *mnt, char *label,
 					char *search)
@@ -649,8 +694,8 @@ static const char * const cmd_filesystem_show_usage[] = {
 	"btrfs filesystem show [options] [<path>|<uuid>|<device>|label]",
 	"Show the structure of a filesystem",
 	"",
-	"-d|--all-devices   show only disks under /dev containing btrfs filesystem",
-	"-m|--mounted       show only mounted btrfs",
+	OPTLINE("-d|--all-devices", "show only disks under /dev containing btrfs filesystem"),
+	OPTLINE("-m|--mounted", "show only mounted btrfs"),
 	HELPINFO_UNITS_LONG,
 	"",
 	"If no argument is given, structure of all present filesystems is shown.",
@@ -707,7 +752,7 @@ static int cmd_filesystem_show(const struct cmd_struct *cmd,
 	if (argc > optind) {
 		search = argv[optind];
 		if (*search == 0)
-			usage(cmd);
+			usage(cmd, 1);
 		type = check_arg_type(search);
 
 		/*
@@ -854,13 +899,13 @@ static const char * const cmd_filesystem_defrag_usage[] = {
 	"btrfs filesystem defragment [options] <file>|<dir> [<file>|<dir>...]",
 	"Defragment a file or a directory",
 	"",
-	"-r                  defragment files recursively",
-	"-c[zlib,lzo,zstd]   compress the file while defragmenting",
-	"-f                  flush data to disk immediately after defragmenting",
-	"-s start            defragment only from byte onward",
-	"-l len              defragment only up to len bytes",
-	"-t size             target extent size hint (default: 32M)",
-	"-v                  deprecated, alias for global -v option",
+	OPTLINE("-r", "defragment files recursively"),
+	OPTLINE("-c[zlib,lzo,zstd]", "compress the file while defragmenting, optional parameter (no space in between)"),
+	OPTLINE("-f", "flush data to disk immediately after defragmenting"),
+	OPTLINE("-s start", "defragment only from byte onward"),
+	OPTLINE("-l len", "defragment only up to len bytes"),
+	OPTLINE("-t size", "target extent size hint (default: 32M)"),
+	OPTLINE("-v", "deprecated, alias for global -v option"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_VERBOSE,
 	"",
@@ -1105,8 +1150,7 @@ static const char * const cmd_filesystem_resize_usage[] = {
 	"on the device 'devid'.",
 	"[kK] means KiB, which denotes 1KiB = 1024B, 1MiB = 1024KiB, etc.",
 	"",
-	"--enqueue         wait if there's another exclusive operation running,",
-	"                  otherwise continue",
+	OPTLINE("--enqueue", "wait if there's another exclusive operation running, otherwise continue"),
 	NULL
 };
 
@@ -1392,7 +1436,12 @@ static DEFINE_COMMAND(filesystem_balance, "balance", cmd_filesystem_balance,
 
 static const char * const cmd_filesystem_mkswapfile_usage[] = {
 	"btrfs filesystem mkswapfile <file>",
-	"Force a sync on a filesystem",
+        "Create a new file that's suitable and formatted as a swapfile.",
+        "Create a new file that's suitable and formatted as a swapfile. Default",
+        "size is 2GiB, minimum size is 40KiB.",
+	"",
+	OPTLINE("-s|--size SIZE", "create file of SIZE (accepting k/m/g/e/p suffix)"),
+	OPTLINE("-U|--uuid UUID", "specify UUID to use, or a special value: clear (all zeros), random, time (time-based random)"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_VERBOSE,
 	HELPINFO_INSERT_QUIET,
@@ -1400,26 +1449,26 @@ static const char * const cmd_filesystem_mkswapfile_usage[] = {
 };
 
 /*
- * Swap signature in the first 4KiB, v2:
+ * Swap signature in the first 4KiB, v2, no label:
  *
  * 00000400 .. = 01 00 00 00 ff ff 03 00  00 00 00 00 cb 70 8e 60
- *                                                    ^^^^^^^^^^^
- *                                                    uuid 4B
+ *                           ^^^^^^^^^^^              ^^^^^^^^^^^
+ *                           page count 4B            uuid 4B
  * 00000420 .. = 1d fb 4e ca be d4 3f 1f  6a 6b 0c 03 00 00 00 00
  *               ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  *               uuid 8B
  * 00000ff0 .. = 00 00 00 00 00 00 53 57  41 50 53 50 41 43 45 32
  *                                  S  W   A  P  S  P  A  C  E  2
  */
-static int write_swap_signature(int fd)
+static int write_swap_signature(int fd, u32 page_count, const uuid_t uuid)
 {
 	int ret;
-	static unsigned char swap[4096] = {
+	static unsigned char swap[SZ_4K] = {
 		[0x400] = 0x01,
-		[0x404] = 0xff,
-		[0x405] = 0xff,
-		[0x406] = 0x03,
+		/* 0x404 .. 0x407 number of pages (little-endian) */
+		/* 0x408 .. 0x40b number of bad pages (unused) */
 		/* 0x40c .. 0x42b UUID */
+		/* Last bytes of the page */
 		[0xff6] = 'S',
 		[0xff7] = 'W',
 		[0xff8] = 'A',
@@ -1431,9 +1480,11 @@ static int write_swap_signature(int fd)
 		[0xffe] = 'E',
 		[0xfff] = '2',
 	};
+	u32 *pages = (u32 *)&swap[0x404];
 
-	uuid_generate(&swap[0x40c]);
-	ret = pwrite(fd, swap, 4096, 0);
+	*pages = cpu_to_le32(page_count);
+	memcpy(&swap[0x40c], uuid, 16);
+	ret = pwrite(fd, swap, SZ_4K, 0);
 
 	return ret;
 }
@@ -1445,16 +1496,20 @@ static int cmd_filesystem_mkswapfile(const struct cmd_struct *cmd, int argc, cha
 	const char *fname;
 	unsigned long flags;
 	u64 size = SZ_2G;
+	u64 page_count;
+	uuid_t uuid;
 
+	uuid_generate(uuid);
 	optind = 0;
 	while (1) {
 		int c;
 		static const struct option long_options[] = {
 			{ "size", required_argument, NULL, 's' },
+			{ "uuid", required_argument, NULL, 'U' },
 			{ NULL, 0, NULL, 0 }
 		};
 
-		c = getopt_long(argc, argv, "s:", long_options, NULL);
+		c = getopt_long(argc, argv, "s:U:", long_options, NULL);
 		if (c < 0)
 			break;
 
@@ -1467,6 +1522,21 @@ static int cmd_filesystem_mkswapfile(const struct cmd_struct *cmd, int argc, cha
 				return 1;
 			}
 			break;
+		case 'U':
+			if (strcmp(optarg, "clear") == 0) {
+				uuid_clear(uuid);
+			} else if (strcmp(optarg, "random") == 0) {
+				uuid_generate(uuid);
+			} else if (strcmp(optarg, "time") == 0) {
+				uuid_generate_time(uuid);
+			} else {
+				ret = uuid_parse(optarg, uuid);
+				if (ret == -1) {
+					error("UUID not recognized: %s", optarg);
+					return 1;
+				}
+			}
+			break;
 		default:
 			usage_unknown_option(cmd, argv);
 		}
@@ -1496,7 +1566,22 @@ static int cmd_filesystem_mkswapfile(const struct cmd_struct *cmd, int argc, cha
 		ret = 1;
 		goto out;
 	}
-	pr_verbose(LOG_INFO, "fallocate to size %llu\n", size);
+	page_count = size / SZ_4K;
+	if (page_count <= 10) {
+		error("file too short");
+		ret = 1;
+		goto out;
+	}
+	/* First file page with header */
+	page_count--;
+	if (page_count > (u32)-1) {
+		error("file too big");
+		ret = 1;
+		goto out;
+	}
+	size = round_down(size, SZ_4K);
+	pr_verbose(LOG_INFO, "fallocate to size %llu, page size %u, %llu pages\n",
+			size, SZ_4K, page_count);
 	ret = fallocate(fd, 0, 0, size);
 	if (ret < 0) {
 		error("cannot fallocate file: %m");
@@ -1504,7 +1589,7 @@ static int cmd_filesystem_mkswapfile(const struct cmd_struct *cmd, int argc, cha
 		goto out;
 	}
 	pr_verbose(LOG_INFO, "write swap signature\n");
-	ret = write_swap_signature(fd);
+	ret = write_swap_signature(fd, page_count, uuid);
 	if (ret < 0) {
 		error("cannot write swap signature: %m");
 		ret = 1;
