@@ -272,7 +272,7 @@ static int __recow_root(struct btrfs_trans_handle *trans, struct btrfs_root *roo
 			goto out;
 		ret = 0;
 		btrfs_item_key_to_cpu(path.nodes[0], &found_key, 0);
-		ASSERT(btrfs_comp_cpu_keys(&key, &found_key) == 0);
+		UASSERT(btrfs_comp_cpu_keys(&key, &found_key) == 0);
 
 next:
 		ret = btrfs_next_leaf(root, &path);
@@ -782,7 +782,7 @@ static int create_uuid_tree(struct btrfs_trans_handle *trans)
 	};
 	int ret = 0;
 
-	ASSERT(fs_info->uuid_root == NULL);
+	UASSERT(fs_info->uuid_root == NULL);
 	root = btrfs_create_tree(trans, fs_info, &key);
 	if (IS_ERR(root)) {
 		ret = PTR_ERR(root);
@@ -1025,7 +1025,6 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 	char *label = NULL;
 	int nr_global_roots = sysconf(_SC_NPROCESSORS_ONLN);
 	char *source_dir = NULL;
-	bool source_dir_set = false;
 
 	cpu_detect_flags();
 	hash_init_accel();
@@ -1172,7 +1171,6 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 			case 'r':
 				free(source_dir);
 				source_dir = strdup(optarg);
-				source_dir_set = true;
 				break;
 			case 'U':
 				strncpy(fs_uuid, optarg,
@@ -1221,11 +1219,11 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 
 	opt_zoned = !!(features.incompat_flags & BTRFS_FEATURE_INCOMPAT_ZONED);
 
-	if (source_dir_set && device_count > 1) {
+	if (source_dir && device_count > 1) {
 		error("the option -r is limited to a single device");
 		goto error;
 	}
-	if (shrink_rootdir && !source_dir_set) {
+	if (shrink_rootdir && source_dir == NULL) {
 		error("the option --shrink must be used with --rootdir");
 		goto error;
 	}
@@ -1246,7 +1244,7 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 	for (i = 0; i < device_count; i++) {
 		file = argv[optind++];
 
-		if (source_dir_set && path_exists(file) == 0)
+		if (source_dir && path_exists(file) == 0)
 			ret = 0;
 		else if (path_is_block_device(file) == 1)
 			ret = test_dev_for_mkfs(file, force_overwrite);
@@ -1352,7 +1350,7 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 	if (opt_zoned) {
 		const int blkid_version =  blkid_get_library_version(NULL, NULL);
 
-		if (source_dir_set) {
+		if (source_dir) {
 			error("the option -r and zoned mode are incompatible");
 			exit(1);
 		}
@@ -1388,7 +1386,7 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 	 *
 	 * This must be done before minimal device size checks.
 	 */
-	if (source_dir_set) {
+	if (source_dir) {
 		int oflags = O_RDWR;
 		struct stat statbuf;
 		int fd;
@@ -1541,12 +1539,14 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 	    btrfs_bg_type_to_tolerated_failures(data_profile))
 		warning("metadata has lower redundancy than data!\n");
 
-	printf("NOTE: several default settings have changed in version 5.15, please make sure\n");
-	printf("      this does not affect your deployments:\n");
-	printf("      - DUP for metadata (-m dup)\n");
-	printf("      - enabled no-holes (-O no-holes)\n");
-	printf("      - enabled free-space-tree (-R free-space-tree)\n");
-	printf("\n");
+	if (bconf.verbose) {
+		printf("NOTE: several default settings have changed in version 5.15, please make sure\n");
+		printf("      this does not affect your deployments:\n");
+		printf("      - DUP for metadata (-m dup)\n");
+		printf("      - enabled no-holes (-O no-holes)\n");
+		printf("      - enabled free-space-tree (-R free-space-tree)\n");
+		printf("\n");
+	}
 
 	mkfs_cfg.label = label;
 	memcpy(mkfs_cfg.fs_uuid, fs_uuid, sizeof(mkfs_cfg.fs_uuid));
@@ -1722,13 +1722,15 @@ raid_groups:
 		goto out;
 	}
 
-	if (source_dir_set) {
-		ret = btrfs_mkfs_fill_dir(source_dir, root, bconf.verbose);
+	if (source_dir) {
+		pr_verbose(LOG_DEFAULT, "Rootdir from:       %s\n", source_dir);
+		ret = btrfs_mkfs_fill_dir(source_dir, root);
 		if (ret) {
 			error("error while filling filesystem: %d", ret);
 			goto out;
 		}
 		if (shrink_rootdir) {
+			pr_verbose(LOG_DEFAULT, "  Shrink:           yes\n");
 			ret = btrfs_mkfs_shrink_fs(fs_info, &shrink_size,
 						   shrink_rootdir);
 			if (ret < 0) {
@@ -1736,6 +1738,8 @@ raid_groups:
 					ret);
 				goto out;
 			}
+		} else {
+			pr_verbose(LOG_DEFAULT, "  Shrink:           no\n");
 		}
 	}
 
@@ -1785,9 +1789,8 @@ raid_groups:
 		btrfs_parse_runtime_features_to_string(features_buf, &features);
 		printf("Runtime features:   %s\n", features_buf);
 #endif
-		printf("Checksum:           %s",
+		printf("Checksum:           %s\n",
 		       btrfs_super_csum_name(mkfs_cfg.csum_type));
-		printf("\n");
 
 		list_all_devices(root);
 
